/**
 * (C) 2011-2012 Alibaba Group Holding Limited.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 */
/**
 * Copyright 2007 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.taobao.common.tedis.support.dictionary;


/**
 * HitǼֵʱصĽֵʱǷһǿյHitʾܵĸ
 * <p>
 *
 * Hit2жϢ
 * <li>ҪĴǷڴʵ: {@link #isHit()}</li>
 * <li>ʵǷԸַͷ: {@link #isUnclosed()}</li>
 * <br>
 * 2ϢǷ񶨵ģ {@link #isUndefined()}true򷵻false. <br>
 * <br>
 *
 * {@link #isHit()}true{@link #getWord()}زҽ{@link #getNext()}һ
 * <br>
 * {@link #isHit()}false{@link #isUnclosed()}true{@link #getNext()}
 * ѯ￪ͷλǰĴ
 * <p>
 *
 * @author Zhiliang Wang [qieqie.wang@gmail.com]
 *
 * @see Dictionary
 * @see BinaryDictionary
 * @see HashBinaryDictionary
 *
 * @since 1.0
 *
 */
public class Hit {

    // -------------------------------------------------

    public final static int UNCLOSED_INDEX = -1;

    public final static int UNDEFINED_INDEX = -2;

    public final static Hit UNDEFINED = new Hit(UNDEFINED_INDEX, null, null);

    // -------------------------------------------------

    /**
     * ĿڴʵеλãֵûиôǱʾ˼(μϾ̬)
     */
    private int index;

    /**
     * ʱʵӦĴ
     */
    private String word;

    /**
     * ʵдʵһʣ{@link #isUnclosed()}Ϊtrueʱӽһ(μע)
     */
    private String next;

    // -------------------------------------------------

    /**
     *
     * @param index
     *            ĿڴʵеλãֵûиôǱʾ˼(μϾ̬)
     * @param word
     *            ʱʵӦĴ
     * @param next
     *            ʵдʵһʣ{@link #isUnclosed()}Ϊtrueʱӽһ(μע)
     */
    public Hit(int index, String word, String next) {
        this.index = index;
        this.word = word;
        this.next = next;
    }

    // -------------------------------------------------

    /**
     * ʱʵӦĴ
     */
    public String getWord() {
        return word;
    }

    /**
     * ĿڴʵеλãֵûиôǱʾ˼(μϾ̬)
     *
     * @return
     */
    public int getIndex() {
        return index;
    }

    /**
     * ʵдʵһʣ{@link #isUnclosed()}Ϊtrueʱӽһ(μע)
     *
     * @return
     */
    public String getNext() {
        return next;
    }

    /**
     * ǷֵмҪĴ
     *
     * @return
     */
    public boolean isHit() {
        return this.index >= 0;
    }

    /**
     * ǷԵǰ￪ͷĴ
     *
     * @return
     */
    public boolean isUnclosed() {
        return UNCLOSED_INDEX == this.index || (this.next != null && this.next.length() >= this.word.length() && this.next.startsWith(word));
    }

    /**
     * ֵûеǰĴ俪ͷĴ
     *
     * @return
     */
    public boolean isUndefined() {
        return UNDEFINED.index == this.index;
    }

    // -------------------------------------------------

    void setIndex(int index) {
        this.index = index;
    }

    void setWord(String key) {
        this.word = key;
    }

    void setNext(String next) {
        this.next = next;
    }

    // -------------------------------------------------

    @Override
    public int hashCode() {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + ((word == null) ? 0 : word.hashCode());
        result = PRIME * result + index;
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final Hit other = (Hit) obj;
        if (word == null) {
            if (other.word != null)
                return false;
        } else if (!word.equals(other.word))
            return false;
        if (index != other.index)
            return false;
        return true;
    }

    public String toString() {
        if (isUnclosed()) {
            return "[UNCLOSED]";
        } else if (isUndefined()) {
            return "[UNDEFINED]";
        }
        return "[" + index + ']' + word;
    }

}
